<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;

use Ems\Services\EstruturaService;

class EstruturaRepository {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getPontoAcesso(string $nrorg, string $nrvinculom, string $tipoEstrutura = EstruturaService::DEFAULT_ESTRUTURA_PONTO_ACESSO) {
        try {
            $sql = <<<SQL
                SELECT PA.IDOBRIGAFOTO,
                       PA.DSLATITUDE,
                       PA.DSLONGITUDE,
                       PA.VRDISTANCIAMAXREF,
                       PA.NRESTRUTURAM
                  FROM GPE_VINCULOM VM 
                  JOIN GPE_VINCULOH VH ON VH.DTMESCOMPETENC = 
                    (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH VHH WHERE VHH.NRVINCULOM = VH.NRVINCULOM AND VHH.NRORG = VH.NRORG)
                    AND VH.NRORG = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                   JOIN GPE_PONTOACESSO PA ON PA.NRESTRUTURAM = VH.$tipoEstrutura
                    AND PA.NRORG = VM.NRORG
                  WHERE VM.NRVINCULOM = :P_NRVINCULOM
                        AND VM.NRORG = :P_NRORG
SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"             => $nrorg,
                    "P_NRVINCULOM"        => $nrvinculom
                )
            );

            $pontoAcesso = $query->fetch();

            if (empty($pontoAcesso) && $tipoEstrutura !== EstruturaService::SECONDARY_ESTRUTURA_PONTO_ACESSO) {
                $pontoAcesso = $this->getPontoAcesso($nrorg, $nrvinculom, EstruturaService::SECONDARY_ESTRUTURA_PONTO_ACESSO);
            }

            return $pontoAcesso;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosEstruturaLegal($nrvinculom, $nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    EM.*,
                    EH.*
                FROM GPE_VINCULOM VM
                JOIN GPE_VINCULOH VH ON VH.DTMESCOMPETENC =
                    (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH VHH WHERE VHH.NRVINCULOM = VH.NRVINCULOM AND VHH.NRORG = VH.NRORG)
                    AND VH.NRORG = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                JOIN ESTRUTURAM EM ON EM.NRORG = VM.NRORG
                    AND EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
                LEFT JOIN ESTRUTURAH EH
                    ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.NRORG = EM.NRORG
                    AND EH.DTMESCOMPETENC =
                        ( SELECT MAX(EHH.DTMESCOMPETENC)
                        FROM ESTRUTURAH EHH
                        WHERE EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                            AND EHH.NRORG = EH.NRORG )
                WHERE VM.NRVINCULOM = :P_NRVINCULOM
                    AND VM.NRORG = :P_NRORG
SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"               => $nrorg,
                    "P_NRVINCULOM"          => $nrvinculom
                )
            );

            return $query->fetchAll()[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEstruturasGerenciaisResp($params) {
        $sql = <<<SQL
            SELECT EM.NRESTRUTURAM
            FROM OPERADOR OP    
            JOIN ESTRUTURAH EH ON EH.NRORG = OP.NRORG

                /* BUSCA O HISTORICO MAIS RECENTE */
                AND EH.DTMESCOMPETENC = (
                    SELECT MAX(DTMESCOMPETENC)
                    FROM ESTRUTURAH EHH
                    WHERE EHH.NRORG = EH.NRORG
                        AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                        AND EHH.DTMESCOMPETENC <= SYSDATE
                    )

                /* FILTRA POR ESTRUTURAS DIRETAMENTE GERENCIADAS PELO OPERADOR */
                AND (
                    INSTR(',' || EH.NRPARCRESPINF || ',', ',' || OP.NRPARCNEGOCIO || ',') > 0 OR
                    INSTR(',' || EH.NRPARCRESPINFADIC || ',', ',' || OP.NRPARCNEGOCIO || ',') > 0
                )
            JOIN ESTRUTURAM EM ON EM.NRORG = OP.NRORG
                AND EM.NRESTRUTURAM = EH.NRESTRUTURAM
                AND EM.IDATIVO = 'S'
                AND (EM.DTFIMVIGENCIA > SYSDATE
                    OR EM.DTFIMVIGENCIA IS NULL)
            WHERE OP.NRORG = :P_NRORG
                AND OP.CDOPERADOR = :P_CDOPERADOR
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $estruturas = $std->fetchAll();

        return $estruturas;
    }

    public function getEstruturasInferiores($params) {
        $sql = <<<SQL
            SELECT
                NRESTRUTURAMREL
            FROM
                HIERARQUIAORG HIER
                JOIN
                ESTRUTURAM EM_INF ON
                    HIER.NRESTRUTURAMREL = EM_INF.NRESTRUTURAM AND
                    HIER.NRORG = EM_INF.NRORG AND
                    EM_INF.IDATIVO = 'S'
                JOIN
                ESTRUTURAM EM_SUP ON
                    HIER.NRESTRUTURAMSUP = EM_SUP.NRESTRUTURAM AND
                    HIER.NRORG = EM_SUP.NRORG AND
                    EM_SUP.IDATIVO = 'S'
            WHERE HIER.NRORG = :P_NRORG
              AND HIER.IDATIVO = 'S'
              AND HIER.NRESTRUTURAMSUP = :P_NRESTRUTURA
              AND (EM_INF.DTFIMVIGENCIA > SYSDATE
                    OR EM_INF.DTFIMVIGENCIA IS NULL)

            --Por razoes de perfomance
            --GROUP BY NRESTRUTURAMREL
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $estruturas = $std->fetchAll();

        return $estruturas;
    }

    public function getNomeETipoEstrutura($params) {
        $sql = <<<SQL
            SELECT
                ESTM.NRESTRUTURAM,
                ('(' || TIPO.NMTIPOESTRUTURA || ') ' || ESTH.NMESTRUTURAH) NMESTRUTURA,
                (CASE
                    WHEN ESTM.NRTIPOESTRUTURA = PARAMS.NRTPESTRUTGEREN THEN 'GERENCIAL'
                    WHEN ESTM.NRTIPOESTRUTURA = PARAMS.NRTPESTRUTLEGAL THEN 'LEGAL'
                    WHEN ESTM.NRTIPOESTRUTURA = 10 THEN 'SINDICAL'
                    ELSE 'OUTRO'
                END) IDTIPOESTRUTURA
            FROM ESTRUTURAM ESTM
            LEFT JOIN ESTRUTURAH ESTH
                 ON ESTH.NRESTRUTURAM = ESTM.NRESTRUTURAM
                AND ESTH.NRORG = ESTM.NRORG
                AND ESTH.DTMESCOMPETENC = (
                    SELECT MAX(DTMESCOMPETENC)
                    FROM ESTRUTURAH ESTH1
                    WHERE ESTH1.NRORG = :P_NRORG
                      AND ESTH1.IDATIVO = 'S'
                      AND ESTH1.NRESTRUTURAM = ESTH.NRESTRUTURAM 
            )
            LEFT JOIN TIPOESTRUTURA TIPO
                 ON ESTM.NRTIPOESTRUTURA = TIPO.NRTIPOESTRUTURA
            LEFT JOIN GPE_PARAMORG PARAMS
                 ON ESTM.NRORG = PARAMS.NRORG
            WHERE ESTM.NRORG        = :P_NRORG
              AND ESTM.NRESTRUTURAM = :P_NRESTRUTURA
              AND ESTM.IDATIVO      = 'S'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $estrutura = $std->fetch();

        return $estrutura;
    }

    public function getEstruturasAcessiveisDP(
        string $nrorg
    ) {
        $sql = <<<SQL
            SELECT EM.*
              FROM ESTRUTURAM EM
              JOIN GPE_PARAMORG PO ON PO.NRORG = EM.NRORG
             WHERE EM.NRORG = :P_NRORG
                   AND (EM.NRTIPOESTRUTURA = PO.NRTPESTRUTGEREN OR EM.NRTIPOESTRUTURA = PO.NRTPESTRUTLEGAL)
                   AND EM.IDATIVO = 'S'
                   AND (EM.DTFIMVIGENCIA > SYSDATE OR EM.DTFIMVIGENCIA IS NULL)
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                "P_NRORG" => $nrorg
            )
        );

        $result = $std->fetchAll();

        return $result;
    }

    public function getEstruturaMestreHistorico(
        string $nrestruturam,
        string $nrorg
    ) {
        $sql = <<<SQL
            SELECT EM.*,
                   EH.*,
                   TE.NMTIPOESTRUTURA,
                   CP_ESTRUT.CDCOMUNICAPARC TELEFONE_ESTRUT,
                   RESP.NMPRINCIPALPARC NMPARCRESPINF,
                   CP_RESP.CDCOMUNICAPARC EMAIL_RESP,
                   DECODE(
                        ENDE.NRENDERECOPARC, NULL, '',
                        LOGRADOURO.DSABREVLOGRA || ' ' || ENDE.DSENDERECO || ' ' || ENDE.NRIMOVELENDERECO ||
                        DECODE(ENDE.DSCOMPLEENDERECO, NULL, '', ', ' || ENDE.DSCOMPLEENDERECO) ||
                        DECODE(ENDE.NRCEPENDERECO, NULL, '', ' - ' || ENDE.NRCEPENDERECO) ||
                        '. ' || M.NMMUNICIPIO || ', ' || ENDE.SGESTADO || ', ' || P.NMPAIS
                   ) ENDERECO
              FROM ESTRUTURAM EM
              LEFT JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG AND EH.NRESTRUTURAM = EM.NRESTRUTURAM AND EH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                  FROM ESTRUTURAH EHH
                 WHERE EHH.NRORG = EH.NRORG
                       AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
               )
              LEFT JOIN TIPOESTRUTURA TE ON
                EM.NRTIPOESTRUTURA = TE.NRTIPOESTRUTURA
              LEFT JOIN COMUNICAPARC CP_ESTRUT ON
                CP_ESTRUT.NRCOMUNICAPARC = (
                    SELECT MAX(NRCOMUNICAPARC)
                    FROM COMUNICAPARC CP1
                    WHERE
                        CP1.NRPARCNEGOCIO = EM.NRPARCNEGOCIO AND
                        CP1.NRORG = EM.NRORG AND
                        CP1.CDFORMACOMU = '01'
                ) AND
                CP_ESTRUT.NRORG = EM.NRORG
              LEFT JOIN PARCNEGOCIO RESP ON
                RESP.NRPARCNEGOCIO = EH.NRPARCRESPINF AND
                RESP.NRORG = EH.NRORG
              LEFT JOIN COMUNICAPARC CP_RESP ON
                CP_RESP.NRCOMUNICAPARC = (
                    SELECT MAX(NRCOMUNICAPARC)
                    FROM COMUNICAPARC CP1
                    WHERE
                        CP1.NRPARCNEGOCIO = RESP.NRPARCNEGOCIO AND
                        CP1.NRORG = RESP.NRORG AND
                        CP1.CDFORMACOMU IN ('05', '09')
                ) AND
                CP_RESP.NRORG = EM.NRORG
              LEFT JOIN ENDERECOPARC ENDE ON
                ENDE.NRENDERECOPARC = (
                    SELECT MAX(NRENDERECOPARC)
                    FROM ENDERECOPARC ENDE1
                    WHERE
                        ENDE1.NRPARCNEGOCIO = EM.NRPARCNEGOCIO AND
                        ENDE1.NRORG = EM.NRORG AND
                        ENDE1.CDTIPOENDERECO = 'PRINCIPAL'
                ) AND
                ENDE.NRORG = EM.NRORG
              LEFT JOIN LOGRADOURO ON
                LOGRADOURO.CDLOGRADOURO = ENDE.CDLOGRADOURO
              LEFT JOIN MUNICIPIO M ON M.CDMUNICIPIO = ENDE.CDMUNICIPIO
              LEFT JOIN PAIS P ON P.CDPAIS = ENDE.CDPAIS
             WHERE EM.NRORG = :P_NRORG
                   AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRESTRUTURAM"    => $nrestruturam
            )
        );

        $result = $std->fetchAll();

        return !empty($result) ? $result[0] : $result;
    }
    
    /**
     * Função responsável por buscar os emails ou o email do responsável informal
     * da estrutura a qual pertence
     */
    public function retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc){
            $sql = "
                SELECT
                        h.nrestruturam,
                        cp.cdcomunicaparc,
                        peh.nmpessoa                       
                    FROM
                        estruturah h                     
                    JOIN
                        comunicaparc cp 
                            on                      cp.nrorg = h.nrorg                        
                            and cp.nrparcnegocio = h.nrparcrespinf                        
                            and CDFORMACOMU = '05'                     
                    JOIN
                        gpe_pessoa pe 
                            on                      pe.nrorg = h.nrorg                          
                            and pe.nrparcnegocio = h.nrparcrespinf                     
                    JOIN
                        gpe_pessoah peh 
                            on  peh.nrorg = pe.nrorg                        
                            and peh.nrpessoa = pe.nrpessoa                        
                            and peh.dtmescompetenc = (
                                SELECT
                                    MAX(DTMESCOMPETENC) 
                            FROM
                                GPE_PESSOAH                                                     
                            WHERE
                                DTMESCOMPETENC <= :p_dtmescompetenc                                                       
                                AND NRORG    = PEH.NRORG                                                       
                                AND NRPESSOA = PEH.NRPESSOA 
                        )                     
                    WHERE
                        h.nrorg = :p_nrorg                        
                        and nrestruturam = :p_nrestrutgeren                        
                        and h.dtmescompetenc = (
                            SELECT
                                MAX(dtmescompetenc)                                                
                            FROM
                                estruturah hh                                               
                            WHERE
                                hh.nrorg = h.nrorg                                                 
                                and hh.nrestruturam = h.nrestruturam                                                 
                                and hh.dtmescompetenc <= :p_dtmescompetenc
                        )

      ";

      $resultado = $this->connection->fetchAll($sql, array(
          "p_nrorg" => $nrorg,
          "p_nrestrutgeren" => $nrestrutgeren,
          "p_dtmescompetenc" => $dtmescompetenc
      ));
      if($resultado)
        return $resultado[0];        
      else
        return $resultado;
    }
    
    public function getEstrutura(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                EH.*,
                EM.NRTPEVENTOCTBL,
                EM.DTESOCIAL,
                EM.NRREGIMEAPUR,
                EM.DTINIVIGENCIA,
                EM.DTFIMVIGENCIA,
                EM.NRTIPOESTRUTURA,
                EM.CDINTESTRUTURA,
                EM.NMESTRUTURAM,
                EM.CDINTESTRUTEXPFIL,
                EM.CDINTESTRUTEXP,
                EM.NRGRUPOESOCIAL,
                PN.CDTIPOINSCRICAO,
                TIPOESTRUTURA.NMTIPOESTRUTURA,
                CT.DSCLASSTRIBUT,
                CA.DSCALENDARIO,
                CN.DSCNAE,
                CN.VRALIQRAT,
                FPAS.DSFPAS,
                PP.NMPRINCIPALPARC AS NMPROPRIEMPR,
                PRI.NMPRINCIPALPARC AS NMPARCRESPINF,
                PRL.NMPRINCIPALPARC AS NMPARCENTISIN,
                PRA.NMPRINCIPALPARC AS NMPARCREGAMB,
                PMB.NMPRINCIPALPARC AS NMPARCMONBIO,
                NJ.DSNATUJURI,
                ESO_PARAMETRO.IDUTILIZAMATRIC 
            FROM
                ESTRUTURAM EM 
                JOIN
                    ESTRUTURAH EH 
                    ON EM.NRESTRUTURAM = EH.NRESTRUTURAM 
                    AND EM.NRORG = EH.NRORG 
                    AND DTMESCOMPETENC = 
                    (
                        SELECT
                            MAX(EHH.DTMESCOMPETENC) 
                        FROM
                            ESTRUTURAH EHH 
                        WHERE
                            EHH.NRORG = :P_NRORG 
                            AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM 
                            AND EHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') 
                    )
                JOIN
                    PARCNEGOCIO PN 
                    ON PN.NRORG = EM.NRORG 
                    AND PN.NRPARCNEGOCIO = EM.NRPARCNEGOCIO 
                JOIN
                    TIPOESTRUTURA 
                    ON EM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA 
                LEFT JOIN
                    CLASSTRIBUT CT 
                    ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT 
                    AND 
                    (
                        CT.NRORG = EM.NRORG 
                        OR CT.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    CALENDARIO CA 
                    ON CA.NRCALENDARIO = EH.NRCALENDARIO 
                    AND 
                    (
                        CA.NRORG = EM.NRORG 
                        OR CA.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    CNAEFISCAL CN 
                    ON CN.CDCNAE = EH.CDCNAE 
                    AND 
                    (
                        CN.NRORG = EM.NRORG 
                        OR CN.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    FPAS 
                    ON FPAS.CDFPAS = EH.CDFPAS 
                    AND 
                    (
                        FPAS.NRORG = EM.NRORG 
                        OR FPAS.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    PARCNEGOCIO PP 
                    ON EH.NRPROPRIEMPR = PP.NRPARCNEGOCIO 
                    AND EM.NRORG = PP.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRI 
                    ON EH.NRPARCRESPINF = PRI.NRPARCNEGOCIO 
                    AND EM.NRORG = PRI.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRL 
                    ON EH.NRPARCENTISIN = PRL.NRPARCNEGOCIO 
                    AND EM.NRORG = PRL.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRA 
                    ON EH.NRPARCREGAMB = PRA.NRPARCNEGOCIO 
                    AND EM.NRORG = PRA.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PMB 
                    ON EH.NRPARCMONBIO = PMB.NRPARCNEGOCIO 
                    AND EM.NRORG = PMB.NRORG 
                LEFT JOIN
                    NATUREZAJURI NJ 
                    ON EH.CDNATUJURI = NJ.CDNATUJURI 
                    AND 
                    (
                        EH.NRORG = NJ.NRORG 
                        OR NJ.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    ESO_PARAMETRO 
                    ON ESO_PARAMETRO.NRESTRUTURAM = EM.NRESTRUTURAM 
                    AND ESO_PARAMETRO.NRORG = EM.NRORG 
            WHERE
                EM.NRORG = :P_NRORG
                AND (
                    EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTURA
                    OR :P_NRTIPOESTRUTURA IS NULL
                )
                AND (
                    EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC
                    OR EM.DTFIMVIGENCIA IS NULL
                )
            ORDER BY
                NRESTRUTURAH ASC,
                NMESTRUTURAH ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $result = $std->fetchAll();
        
        return $result;
    }

    public function getEstruturaAdicional(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                EH.*,
                EM.NRTPEVENTOCTBL,
                EM.DTESOCIAL,
                EM.NRREGIMEAPUR,
                EM.DTINIVIGENCIA,
                EM.DTFIMVIGENCIA,
                EM.NRTIPOESTRUTURA,
                EM.CDINTESTRUTURA,
                EM.NMESTRUTURAM,
                EM.CDINTESTRUTEXPFIL,
                EM.CDINTESTRUTEXP,
                EM.NRGRUPOESOCIAL,
                PN.CDTIPOINSCRICAO,
                TIPOESTRUTURA.NMTIPOESTRUTURA,
                CT.DSCLASSTRIBUT,
                CA.DSCALENDARIO,
                CN.DSCNAE,
                CN.VRALIQRAT,
                FPAS.DSFPAS,
                PP.NMPRINCIPALPARC AS NMPROPRIEMPR,
                PRI.NMPRINCIPALPARC AS NMPARCRESPINF,
                PRL.NMPRINCIPALPARC AS NMPARCENTISIN,
                PRA.NMPRINCIPALPARC AS NMPARCREGAMB,
                PMB.NMPRINCIPALPARC AS NMPARCMONBIO,
                NJ.DSNATUJURI,
                ESO_PARAMETRO.IDUTILIZAMATRIC 
            FROM
                ESTRUTURAM EM 
                JOIN
                    ESTRUTURAH EH 
                    ON EM.NRESTRUTURAM = EH.NRESTRUTURAM 
                    AND EM.NRORG = EH.NRORG 
                    AND DTMESCOMPETENC = 
                    (
                        SELECT
                            MAX(EHH.DTMESCOMPETENC) 
                        FROM
                            ESTRUTURAH EHH 
                        WHERE
                            EHH.NRORG = :P_NRORG 
                            AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM 
                            AND EHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') 
                    )
                JOIN
                    PARCNEGOCIO PN 
                    ON PN.NRORG = EM.NRORG 
                    AND PN.NRPARCNEGOCIO = EM.NRPARCNEGOCIO 
                JOIN
                    TIPOESTRUTURA 
                    ON EM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA 
                LEFT JOIN
                    CLASSTRIBUT CT 
                    ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT 
                    AND 
                    (
                        CT.NRORG = EM.NRORG 
                        OR CT.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    CALENDARIO CA 
                    ON CA.NRCALENDARIO = EH.NRCALENDARIO 
                    AND 
                    (
                        CA.NRORG = EM.NRORG 
                        OR CA.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    CNAEFISCAL CN 
                    ON CN.CDCNAE = EH.CDCNAE 
                    AND 
                    (
                        CN.NRORG = EM.NRORG 
                        OR CN.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    FPAS 
                    ON FPAS.CDFPAS = EH.CDFPAS 
                    AND 
                    (
                        FPAS.NRORG = EM.NRORG 
                        OR FPAS.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    PARCNEGOCIO PP 
                    ON EH.NRPROPRIEMPR = PP.NRPARCNEGOCIO 
                    AND EM.NRORG = PP.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRI 
                    ON EH.NRPARCRESPINF = PRI.NRPARCNEGOCIO 
                    AND EM.NRORG = PRI.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRL 
                    ON EH.NRPARCENTISIN = PRL.NRPARCNEGOCIO 
                    AND EM.NRORG = PRL.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PRA 
                    ON EH.NRPARCREGAMB = PRA.NRPARCNEGOCIO 
                    AND EM.NRORG = PRA.NRORG 
                LEFT JOIN
                    PARCNEGOCIO PMB 
                    ON EH.NRPARCMONBIO = PMB.NRPARCNEGOCIO 
                    AND EM.NRORG = PMB.NRORG 
                LEFT JOIN
                    NATUREZAJURI NJ 
                    ON EH.CDNATUJURI = NJ.CDNATUJURI 
                    AND 
                    (
                        EH.NRORG = NJ.NRORG 
                        OR NJ.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                    ESO_PARAMETRO 
                    ON ESO_PARAMETRO.NRESTRUTURAM = EM.NRESTRUTURAM 
                    AND ESO_PARAMETRO.NRORG = EM.NRORG 
            WHERE
                EM.NRORG = :P_NRORG
                AND EM.NRTIPOESTRUTURA NOT IN (1, 6, 10)
                AND (
                    EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC
                    OR EM.DTFIMVIGENCIA IS NULL
                )
            ORDER BY
                NRESTRUTURAH ASC,
                NMESTRUTURAH ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $result = $std->fetchAll();
        
        return $result;
    }

    public function getAniversariantesEstrutura(
        array $params
    ): array {
        $sql = <<<SQL
SELECT * FROM
(
SELECT DISTINCT
            TO_CHAR(PEH.DTNASCPESSOA, 'DD/MM') DTANIVERSARIO,
            VM.NMVINCULOM,
            VM.NRVINCULOM,
            EH.NMESTRUTURAH,
            OP.DSEMAILOPER
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                                AND VH.NRVINCULOM = VM.NRVINCULOM
                                AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                          WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                                            AND NRORG      = VH.NRORG
                                                            AND NRVINCULOM = VH.NRVINCULOM )
            /*Estrutura Gerencial*/
            JOIN ESTRUTURAM EMG ON EMG.NRORG        = VM.NRORG
                               AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN --!
            JOIN ESTRUTURAH EH ON EH.NRORG = EMG.NRORG
                              AND EH.NRESTRUTURAH = (
                                  SELECT EH2.NRESTRUTURAH
                                  FROM ESTRUTURAH EH2
                                  WHERE EH2.NRESTRUTURAM = EMG.NRESTRUTURAM
                                    AND EH2.NRORG = EMG.NRORG
                                    AND EH2.DTMESCOMPETENC = (
                                        SELECT MAX(DTMESCOMPETENC)
                                        FROM ESTRUTURAH EH3
                                        WHERE EH3.NRESTRUTURAM = EMG.NRESTRUTURAM
                                          AND EH3.NRORG = EMG.NRORG
                                    )
                              )
            JOIN GPE_PESSOA PE ON PE.NRORG    = VM.NRORG
                              AND PE.NRPESSOA = VM.NRPESSOA
            JOIN GPE_PESSOAH PEH ON PEH.NRORG    = VM.NRORG
                                     AND PEH.NRPESSOA = PE.NRPESSOA
                                     AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                                WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                                                  AND NRORG    = PEH.NRORG
                                                                  AND NRPESSOA = PEH.NRPESSOA )
                                    AND SUBSTR(:P_DTMESCOMPETENC, 4, 2) = TO_CHAR(PEH.DTNASCPESSOA, 'MM')
                                    AND PEH.DTNASCPESSOA IS NOT NULL
            -- /*Ocupação*/
            -- LEFT JOIN GPE_OCUPACAOH OH ON OH.NRORG       = VM.NRORG
            --                           AND OH.NROCUPACAOM = NVL( VH.NRCARGO, VH.NRFUNCAO )
            --                           AND OH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
            --                                                     WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
            --                                                       AND NRORG       = OH.NRORG
            --                                                       AND NROCUPACAOM = OH.NROCUPACAOM )
            LEFT JOIN OPERADOR OP ON OP.NRORG = VM.NRORG
                                      AND OP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            WHERE VM.NRORG = :P_NRORG
            AND ( VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR VM.DTRESCISAOVINC IS NULL )
            ORDER BY DTANIVERSARIO ASC
) P
WHERE
    EXISTS (
        SELECT 1
        FROM GPE_VINCULOH VH
        WHERE
            VH.DTMESCOMPETENC = (
                SELECT MAX(VH1.DTMESCOMPETENC)
                FROM GPE_VINCULOH VH1
                WHERE
                    VH1.NRVINCULOM = P.NRVINCULOM AND
                    VH.NRORG = :P_NRORG AND
                    VH1.DTMESCOMPETENC <= SYSDATE
            ) AND
            VH.NRVINCULOM = P.NRVINCULOM AND
            VH.NRORG = :P_NRORG AND
            (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0
    )
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();
        return $result;
    }
}