<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Util\{ DateUtil, Repositories };
use Ems\Util\Utils;

class MinhaHistoriaRepository {
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function insereSolicitacao($nrsolicitacao, $nrtiposolicitacao, $nrvinculom, $nrorg, $cdoperador, $dsobservacao, $dspath = null) {

        $nrmodeloaprovacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getModeloSolicitacaoBeforeInsert(
            $nrorg, $nrtiposolicitacao, $nrvinculom, Utils::getParamters()->getDtmescomptencDiaCorrente()
        );

        $sql = <<<SQL
            INSERT INTO GPE_SOLICITACAO
                (NRORG, NRSOLICITACAO, NRTIPOSOLICITACAO, NRVINCULOM, DTSOLICITACAO, DSOBSSOLICITACAO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTULTATU, CDOPERULTATU, NRORGULTATU, DSPATH, NRMODELOAPROVSOLI)
            VALUES
                (:NRORG, :NRSOLICITACAO, :NRTIPOSOLICITACAO, :NRVINCULOM, :DTSOLICITACAO, :DSOBSSOLICITACAO, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO, :DTULTATU, :CDOPERULTATU, :NRORGULTATU, :DSPATH, :NRMODELOAPROVSOLI)
SQL;

        return $this->connection->prepare($sql)
            ->execute(array(
                "NRORG"             => $nrorg,
                "NRSOLICITACAO"     => $nrsolicitacao,
                "NRTIPOSOLICITACAO" => $nrtiposolicitacao,
                "NRVINCULOM"        => $nrvinculom,
                "DTSOLICITACAO"     => DateUtil::getDataAtual(true)->format('d/m/Y'),
                "DSOBSSOLICITACAO"  => $dsobservacao,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
                "DTULTATU"          => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdoperador,
                "DSPATH"            => $dspath,
                "NRMODELOAPROVSOLI" => $nrmodeloaprovacao,
            )
        );
    }
    
    public function insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, $dschaveitemsoli, $dslabelitemsoli, $dsvaloritemsoli, $nrorg, $cdoperador) {

        $query = 'INSERT INTO GPE_ITEMSOLICITACAO
                      (NRORG, NRITEMSOLICITACAO, NRSOLICITACAO, DSCHAVEITEMSOLI, DSLABELITEMSOLI, DSVALORITEMSOLI, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                    VALUES
                      (:NRORG, :NRITEMSOLICITACAO, :NRSOLICITACAO, :DSCHAVEITEMSOLI, :DSLABELITEMSOLI, :DSVALORITEMSOLI, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)';

        return $this->connection->prepare($query)
            ->execute(array(
                "NRORG"             => $nrorg,
                "NRITEMSOLICITACAO" => $nritemsolicitacao,
                "NRSOLICITACAO"     => $nrsolicitacao,
                "DSCHAVEITEMSOLI"   => $dschaveitemsoli,
                "DSLABELITEMSOLI"   => $dslabelitemsoli,
                "DSVALORITEMSOLI"   => $dsvaloritemsoli,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
            )
        );
    }

    
}