<?php

namespace Tests;

use Ems\Util\Utils;

use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Util {
    public const DEFAULT_PK = 999999999999;
    public const DEFAULT_ORG = 998;
    public const DEFAULT_ORG_PADRAO = 0;

    /**
     * Retorna a entidade de manipulacao de instancias
     * @return InstanceManager
     */
    public static function getInstanceManager(): InstanceManager {
        return InstanceManager::getInstance();
    }

    /**
     * Retorna a entidade de controle das demais entidades
     * @return \Doctrine\ORM\EntityManagerInterface
     */
    public static function getEntityManager(): \Doctrine\ORM\EntityManagerInterface {
        return self::getInstanceManager()
            ->getService('entitymanager');
    }

    /**
     * Retorna a entidade de conexao com a base de dados
     * @return \Doctrine\DBAL\Connection
     */
    public static function getConnection(): \Doctrine\DBAL\Connection {
        return self::getEntityManager()
            ->getConnection();
    }

    /**
     * Executa um comando SQL na base de dados. Retorna as informacoes que a entidade \Doctrine\DBAL\Connection
     * retornar
     * @return int|string
     */
    public static function executeStatementOnConnection(string $sqlStatement) {
        return Util::getConnection()->executeStatement($sqlStatement);
    }

    /**
     * Inicia uma transacao na base de dados
     * @return void
     */
    public static function beginTransactionOnConnection(): void {
        Util::getConnection()->beginTransaction();
    }

    /**
     * Sincroniza as informacoes na base de dados
     * @return void
     */
    public static function commitOnConnection(): void {
        Util::getConnection()->commit();
    }

    /**
     * Desabilita o auto commit na base de dados. Apos a utilizacao da rotina sera necessario
     * acionar o "commit" da \Doctrine\DBAL\Connection para a persistencia dos dados
     * @return void
     */
    public static function disableAutoCommitOnConnection(): void {
        Util::getConnection()->setAutoCommit(false);
    }

    /**
     * Habilita o auto commit na base de dados
     * @return void
     */
    public static function enableAutoCommitOnConnection(): void {
        Util::getConnection()->setAutoCommit(true);
    }

    /**
     * Define a sessao padrao para os casos de teste
     * @return void
     */
    public static function setDefaultTestSession(): void {
        $session = self::getInstanceManager()->getService('session');
        $session->set("NRORG", self::DEFAULT_ORG);
        $session->set("NRORGTRAB", self::DEFAULT_ORG);
        $session->set("NRORGPADRAO", self::DEFAULT_ORG_PADRAO);
    }

    /**
     * Atualiza as informacoes da tabela de novo codigo. Recomenda-se utilizar
     * essa rotina apos a exclusao de dados criados pelos testes
     * @return void
     */
    public static function updateAllNovoCodigoTables(): void {
        $allTablesUsed = [
            'PTO_ALTEBANCOHORAS' => 'NRALTEBANCOHORAS', 'PTO_PROGOCORRENCIA' => 'NRPROGOCORRENCIA', 'GPE_ALTEESCALA' => 'NRALTEESCALA',
            'PTO_RESUMOSITUFREQ' => 'NRRESUMOSITUFREQ', 'GPE_APURAFREQUENCIA' => 'NRAPURAFREQUENCIA', 'PTO_PERIODOAPURACAO' => 'NRPERIODOAPURACAO',
            'PTO_TIPOPERIODOAPURACAO' => 'NRTIPOPERIODOAPURACAO', 'PTO_TIPOAPURACAOFREQ' => 'NRTIPOAPURACAO', 'GPE_MARCACAOPONTO' => 'NRMARCACAOPONTO',
            'PTO_ASSOCSITUFREQ' => 'NRASSOCSITUFREQ', 'PTO_SITUFREQPARAM' => 'NRSITUFREQPARAM', 'PTO_SITUFREQUENCIA' => 'NRSITUFREQUENCIA',
            'GPE_MOVIMENTACAO' => 'NRMOVIMENTACAO', 'GPE_VINCULOH' => 'NRVINCULOH', 'GPE_VINCULOM' => 'NRVINCULOM', 'GPE_PESSOAH' => 'NRPESSOAH',
            'GPE_PESSOA' => 'NRPESSOA', 'PARCNEGOCIO' => 'NRPARCNEGOCIO', 'GPE_INTERVALOSHORARIO' => 'NRINTERVALOHORARIO',
            'GPE_HORARIOTURNO' => 'NRHORARIOTURNO', 'GPE_TURNO' => 'NRTURNO', 'GPE_ESCALATRABH' => 'NRESCALATRABH',
            'GPE_ESCALATRABM' => 'NRESCALATRABM', 'GPE_HORDIAH' => 'NRHORDIAH', 'GPE_HORDIAM' => 'NRHORDIAM',
        ];

        foreach ($allTablesUsed as $tableName => $tableKey) {
            Utils::updateNovoCodigo($tableName, true, $tableKey);
        }

        self::commitOnConnection();
    }
}